# -*- coding: utf-8 -*-

import mimetypes

from django import forms
from django.core.files.images import ImageFile, get_image_dimensions
from django.db.models.fields.files import FieldFile, ImageField

from south.modelsinspector import add_introspection_rules


class UploadFieldFile(ImageFile, FieldFile):

    def save(self, name, content, save=True):
        type, format = self.mime()
        if type == 'image' and format in ('jpeg', 'png', 'gif', 'tiff'):
            self._dimensions_cache = get_image_dimensions(content)
            if self.field.width_field:
                setattr(self.instance, self.field.width_field, self.width)
            if self.field.height_field:
                setattr(self.instance, self.field.height_field, self.height)
        super(UploadFieldFile, self).save(name, content, save)

    def delete(self, save=True):
        # Clear the image dimensions cache
        if hasattr(self, '_dimensions_cache'):
            del self._dimensions_cache
        super(UploadFieldFile, self).delete(save)

    def mime(self):
        try:
            # p1, p2 = mimetypes.guess_type(self.name)[0].split('/')
            # return p1 == 'image' and p2 in ('jpeg', 'png', 'gif', 'tiff')
            return mimetypes.guess_type(self.name)[0].split('/')
        except AttributeError:
            pass
        return False

    def is_swf(self):
        try:
            return mimetypes.guess_type(self.name)[0] ==\
                'application/x-shockwave-flash'
        except AttributeError:
            return False


class UploadField(ImageField):

    attr_class = UploadFieldFile

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.FileField}
        defaults.update(kwargs)
        return super(ImageField, self).formfield(**defaults)
        # return super(UploadField, self).formfield(**defaults)
add_introspection_rules([], ["^meringue\.fields\.UploadField"])


class PreviewImageField(ImageField):

    def formfield(self, **kwargs):
        kwargs['widget'] = PreviewImageFileInput
        defaults = kwargs
        return super(PreviewImageField, self).formfield(**defaults)



# -*- coding: utf-8 -*-

__VERSION__ = (
    (0, 3, 1),
    # ('a', 4),
    # ('dev', 7)
)

try:
    from verlib import NormalizedVersion
    version = str(NormalizedVersion.from_parts(*__VERSION__))
except ImportError:
    version = '.'.join([str(j) for i in __VERSION__ for j in i])



# -*- coding: utf-8 -*-

import logging
try:
    from verlib import NormalizedVersion
except ImportError:
    pass


def get_version(version):
    try:
        return str(NormalizedVersion.from_parts(*version))
    except NameError:
        logging.info(u'for better install verlib' )
        return '.'.join([str(j) for i in version for j in i])



# -*- coding: utf-8 -*-


import logging
import re
import sys
import os

from django.conf import settings
from django.template import Library
from django.utils import timezone
from django.utils.module_loading import import_string

from meringue import settings as m_settings
from meringue.errors import FileNotFindError


register = Library()

@register.simple_tag
def cop_year():
    '''
    Шаблонный тег возвращает диапозон годов для футера в формате:
        YYYY-YYYY
        YYYY

    Год старта проекта указывается в settings проекта - START_YEAR
    '''

    year = timezone.now().year
    return year == m_settings.START_YEAR and year or '%d&mdash;%d' %\
        (m_settings.START_YEAR, year)


class PutStatic:

    def __init__(self, path, type, load_min=True, load=True, file=None,
                 fix_relative_path=True, fix_map_link=True):
        self.dir, self.filename = os.path.split(path)
        self.load_min = load_min
        self.type = type
        self.file = file
        self.fix_relative_path = fix_relative_path
        self.fix_map_link = fix_map_link

        if load and not file:
            self.file, self.minimal = self._load_static_file()
        else:
            self.file = file

    def _load_file(self, path):
        for backend_finder in settings.STATICFILES_FINDERS:
            finder = import_string(backend_finder)
            match = finder().find(path)
            if match:
                break

        if not match:
            raise FileNotFindError(os.path.split(path)[1])
        return open(match, 'r')

    def _get_min_filename(self):
        return re.sub(
            re.compile(ur'^(?P<name>[\w\W]*).(?P<type>css|js)$', re.UNICODE),
            lambda m: '%s.min.%s' % (m.groupdict()['name'],
                                     m.groupdict()['type']),
            self.filename
        )

    def _load_static_file(self):
        file = None

        if self.load_min:
            try:
                file = self._load_file(os.path.join(
                    self.dir,
                    self._get_min_filename()
                ))
                minimal = True
            except FileNotFindError:
                pass

        if not file:
            file = self._load_file(os.path.join(self.dir, self.filename))
            minimal = False

        return file, minimal

    def _fix_relative_path(self, text):
        if self.type == 'css':
            result = re.sub(
                re.compile(ur'url\(("|\'|\ |)(\.\.\/)(?P<path>[^\"\'\)]+)("|\'|)\)',
                           re.MULTILINE | re.UNICODE),
                lambda m: 'url(%s%s)' % (settings.STATIC_URL,
                                         m.groupdict()['path']),
                text
            )
        elif self.type == 'js':
            result = text
        return result

    def _fix_map_link(self, text):
        result = re.sub(
            re.compile(ur'sourceMappingURL=(?P<scheme>https:\/\/|http:\/\/|\/\/)?(?P<path>[\w\.]+)',
                       re.UNICODE),
            lambda m: 'sourceMappingURL=%s%s' % (settings.STATIC_URL,
                                                 m.groupdict()['path']),
            text
        )
        return result

    def read(self):
        if not self.file:
            self.file, self.minimal = self._load_static_file()

        result = self.file.read()

        if self.fix_relative_path:
            result = self._fix_relative_path(result)

        if self.fix_map_link:
            result = self._fix_map_link(result)

        return result


@register.simple_tag
def put_css(path):
    '''
        Тег выводит содержимое css файла инлайном при этом приобразуя
        относительные пути, при этом пытается вывести минифицированный
        вариант файла
    '''
    result_wrapper = "<style>%s</style>"

    try:
        file = PutStatic(path, 'css')  # , load_min=not settings.DEBUG)
    except FileNotFindError, error:
        if settings.TEMPLATE_DEBUG:
            return u'<script type=\"text/javascript\">console.log(\"файл\
 %s ненайден\"); </script>' % path
        raise error

    return result_wrapper % file.read()


@register.simple_tag
def put_js(path):
    '''
        Тег выводит содержимое js файла инлайном при этом приобразуя
        относительные пути, при этом пытается вывести минифицированный
        вариант файла
    '''
    result_wrapper = "<script type=\"text/javascript\">%s</script>"

    try:
        file = PutStatic(path, 'js')  # , load_min=not settings.DEBUG)
    except FileNotFindError, error:
        if settings.TEMPLATE_DEBUG:
            return u'<script type=\"text/javascript\">console.log(\"файл\
 %s ненайден\"); </script>' % path
        raise error

    return result_wrapper % file.read()


@register.simple_tag
def put_reset():
    '''
    Шаблонный тег выводит содержимое файла reset.css заключённое в тег style
    '''
    return '<style>%s</style>' % open(m_settings.path('meringue/static/css/reset%s.css' % '' if settings.DEBUG else '.min'), 'r').read()



# -*- coding: utf-8 -*-

from django.conf import settings


def base(request):
    return {
        'SITE_NAME': settings.SITE_NAME,
        'TEMPLATE_DEBUG': settings.TEMPLATE_DEBUG,
        'DEBUG': settings.DEBUG,
    }



# -*- coding:utf-8 -*-

from django import forms


class form_fieldsets(object):
    '''
        mixin разбивает форму на несколько групп
        группы указываются в виде списка в Meta.fieldsets.
        элементы списка - словари с ключами fields и title, содержащие список
        полей и название группы соответственно
    '''

    def fieldsets(self):
        fieldsets = getattr(self.Meta, 'fieldsets', [{'fields': self.fields.keys(), 'title': 'main'}])
        result = list()
        for fieldset in fieldsets:
            result.append({
                'fields': [forms.forms.BoundField(self, self.fields[f], f)
                           for f in fieldset['fields'] if f in self.fields],
                'title': fieldset.get('title', None)
            })
        return result


class BaseFormSet(forms.BaseFormSet):
    """
        добавил возможность указать префикс для всего набора форм включая
        управляющие поля
    """
    def __init__(self, *args, **kwargs):
        prefix = self.prefix
        super(BaseFormSet, self).__init__(*args, **kwargs)
        if prefix:
            self.prefix = prefix


def formset_factory(form, formset=BaseFormSet, extra=1, can_order=False,
                    can_delete=False, max_num=None, validate_max=False,
                    min_num=None, validate_min=False, prefix=None):
    """
        помимо всего передаём префикс
    """
    if min_num is None:
        min_num = forms.formsets.DEFAULT_MIN_NUM
    if max_num is None:
        max_num = forms.formsets.DEFAULT_MAX_NUM
    # hard limit on forms instantiated, to prevent memory-exhaustion attacks
    # limit is simply max_num + DEFAULT_MAX_NUM (which is 2*DEFAULT_MAX_NUM
    # if max_num is None in the first place)
    absolute_max = max_num + forms.formsets.DEFAULT_MAX_NUM
    attrs = {'form': form, 'extra': extra,
             'can_order': can_order, 'can_delete': can_delete,
             'min_num': min_num, 'max_num': max_num,
             'absolute_max': absolute_max, 'validate_min': validate_min,
             'validate_max': validate_max, 'prefix': prefix}
    return type(form.__name__ + str('FormSet'), (formset,), attrs)



# -*- coding:utf-8 -*-


class UrlPatterns(object):

    '''
        Возвращает патерн урлов с прописанным приложением и пространством
    имён
        Приложение и пространство имён по умолчанию равно названию приложения
    приведённому к нижнему регистру и заменёнными точками на нижние
    подчёркивания (если приложение часть иного модуля)

        При инициализации можно задать свои namespace и app_name
    '''

    def __init__(self, namespace=None, app_name=None):
        self.app_name = app_name or '_'.join(self.__module__.split('.')[:-1])\
            .lower()
        self.namespace = namespace or self.app_name

    def __call__(self):
        return self.get_urlpatterns(), self.app_name, self.namespace

    def get_urlpatterns(self):
        return self.urlpatterns



# -*- coding: utf-8 -*-

import re

from django.conf import settings
from django.contrib.sites.models import Site
from django.shortcuts import get_object_or_404


hosts_middleware = 'django_hosts.middleware.HostsMiddleware'
site_middleware = 'meringue.middleware.SiteMiddleware'


class SiteMiddleware:
    '''
        Подставляет SITE_ID и SITE_NAME при запросе, используется при
    связке с django-hosts
    '''

    def process_request(self, request):
        middlewares = list(settings.MIDDLEWARE_CLASSES)
        if (middlewares.index(hosts_middleware) >
                middlewares.index(site_middleware)):
            raise ImproperlyConfigured('The django_hosts and simple\
middlewares are in the wrong order. Make sure %r comes before %r in the\
MIDDLEWARE_CLASSES setting.' % (hosts_middleware, site_id_middleware))
        r = r'^(.[^:]+)'
        host = re.findall(r, request.get_host())[0]
        site = get_object_or_404(Site, domain=host)
        settings.SITE_ID = site.id
        settings.SITE_NAME = site.name
        # Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko



# -*- coding: utf-8 -*-

import logging
import re

from django import forms
from django import template
from django.utils.safestring import mark_safe


register = template.Library()

def get_name(field):
    caption = field.widget.__class__.__name__
    r = ur'([A-Z]{1})([a-z]+)'
    p = re.compile(r)
    caption = p.sub(lambda m: '_'+m.group().lower(), caption)
    return caption


@register.filter
def field_render_classes(boundfield):
    """
        выводит список параметров через пробел
            required - если обязательное поле
            errors - если есть ошибки
            valid - если валидное
            meringue-<field_name> - идентификатор
    """

    result = ''
    if boundfield.field.required:
        result += ' required'
    if boundfield.errors:
        result += ' errors'
    if not boundfield.errors and boundfield.form.errors and boundfield.value:
        result += ' valid'
    result += ' meringue-'+get_name(boundfield.field)[1:]
    return result


class FieldRender(object):

    def __init__(self, boundfield):
        self.field = boundfield
        self.caption = get_name(boundfield.field)

    def render(self):
        result = getattr(self, self.caption+'_render', self._default_with_label_render)
        return result()

    # def _default_render(self):
    #     return self.field

    def _default_with_label_render(self):
        result = u'<label for=\"id_%s\" >%s</label>%s' % (
            self.field.html_name,
            unicode(self.field.label),
            self.field
        )
        return mark_safe(result)

    # def _customize_select_render(self):
    #     return self._default_with_label_render()

    # def _input_with_label_render(self):
    #     return self._default_with_label_render()

    # def _checkbox_select_multiple_render(self):
    #     return self._default_with_label_render()

    # def _custom_range_slider_render(self):
    #     return self._default_with_label_render()

    # def _checkbox_input_render(self):
    #     result = u'%s<label for=\"id_%s\" >%s</label>' % (
    #         self.field,
    #         self.field.html_name,
    #         unicode(self.field.label),
    #     )
    #     return mark_safe(result)


@register.filter
def field_render(boundfield):
    """
        Рендерит поле в соответствии с правилами
    """

    result = FieldRender(boundfield)
    return result.render()



# -*- coding: utf-8 -*-

from functools import update_wrapper

from django.http import HttpResponseRedirect, HttpResponseForbidden


class CheckAccount(object):
    """
    "По образу и подобию django.contrib.auth.decorators._CheckLogin"(c)
    """

    def __init__(self, view_func, test_func, redirect_url=None):
        self.view_func = view_func
        self.test_func = test_func
        self.redirect_url = redirect_url
        update_wrapper(self, view_func)

    def __get__(self, obj, cls=None):
        view_func = self.view_func.__get__(obj, cls)
        return CheckAccount(view_func, self.test_func, self.redirect_url)

    def __call__(self, request, *args, **kwargs):
        if self.test_func(request.user):
            return self.view_func(request, *args, **kwargs)
        if self.redirect_url:
            return HttpResponseRedirect(self.redirect_url)
        return HttpResponseForbidden()


def anonymous_required(function=None, redirect_url=None):
    t = lambda u: u.is_anonymous()
    decorator = lambda f: CheckAccount(f, t, redirect_url)
    if function:
        return decorator(function)
    return decorator



# -*- coding:utf-8 -*-

from django import forms
from django.db import models
from django.utils.encoding import force_text
from django.utils.html import conditional_escape, format_html
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from meringue.utils.thumbnails import get_thumbnail


class PreviewImageFileInput(forms.widgets.FileInput):
    '''
    TO-DO:
        информация о изображении
        изображение при инициализации
        при загрузке изображения сразу же его выводить в превью (js)
    '''

    initial_text = _(u'Currently')
    input_text = _(u'Change')
    clear_checkbox_label = _(u'Clear')

    preview = '<img src=\"%(src)s\" data-src=\"%(data_src)s\" >'
    template_with_initial = u'<p class="file-upload">%(preview)s<br />\
        %(initial_text)s: %(initial)s %(clear_template)s<br />\
        %(input_text)s: %(input)s</p>'
    template_with_clear = u'<span class="clearable-file-input">\
        %(clear)s <label for="%(clear_checkbox_id)s">\
        %(clear_checkbox_label)s</label></span>'
    url_markup_template = u'<a href="{0}">{1}</a>'

    def clear_checkbox_name(self, name):
        """
        Given the name of the file input, return the name of the clear
        checkbox input.
        """
        return name + u'-clear'

    def clear_checkbox_id(self, name):
        """
        Given the name of the clear checkbox input, return the HTML id
        for it.
        """
        return name + u'_id'

    def get_preview(self, value, size):
        if value:
            data = {
                'src': get_thumbnail(value.path, [
                    's:%dx%d' % size,
                    'resize',
                    'crop'
                ]),
                'data_src': value.url,
                # 'alt': value.title,
            }
        else:
            data = {
                'src': '',
                'data_src': '',
            }
        return self.preview % data

    def render(self, name, value, size=(100, 100), attrs=None):
        substitutions = {
            'initial_text': self.initial_text,
            'preview': self.get_preview(value, size),
            'input_text': self.input_text,
            'clear_template': u'',
            'clear_checkbox_label': self.clear_checkbox_label,
        }
        template = '%(input)s'
        substitutions['input'] = super(PreviewImageFileInput, self).\
            render(name, value, attrs)

        if value and hasattr(value, "url"):
            template = self.template_with_initial
            substitutions['initial'] = format_html(
                self.url_markup_template,
                value.url,
                force_text(value)
            )
            if not self.is_required:
                checkbox_name = self.clear_checkbox_name(name)
                checkbox_id = self.clear_checkbox_id(checkbox_name)
                substitutions['clear_checkbox_name'] =\
                    conditional_escape(checkbox_name)
                substitutions['clear_checkbox_id'] =\
                    conditional_escape(checkbox_id)
                substitutions['clear'] = forms.CheckboxInput().render(
                    checkbox_name,
                    False,
                    attrs={'id': checkbox_id}
                )
                substitutions['clear_template'] =\
                    self.template_with_clear % substitutions

        return mark_safe(template % substitutions)



# -*- coding:utf-8 -*-

'''
Библиотека с функциями унификации данных разных форматов
'''

try:
    import phonenumbers as pn
    from phonenumbers.phonenumberutil import NumberParseException
except ImportError:
    pass


def unify_email(origin_email):
    if not origin_email or not '@' in origin_email[1:-3]:
        return None
    user, domain = origin_email.strip().rsplit('@', 1)
    user = user.split('+')[0]
    email = '@'.join([user, domain])
    email = email.lower()
    return email


def unify_phone(origin_phone):
    try:
        try:
            phone = pn.parse(origin_phone)
        except NumberParseException:
            phone = pn.parse(origin_phone, 'RU')
        return pn.format_number(phone, pn.PhoneNumberFormat.INTERNATIONAL)
    except NameError:
        raise ImportError('django-phonenumber-field module is not available')



# -*- coding: utf-8 -*-

from django.template import Library

from meringue.utils.thumbnails import get_thumbnail


register = Library()


@register.filter
def thumbnail(filename, args=''):
    '''
        Фильтр обработки изображений, применяется к локальнуму адресу
    изображения (путь до файла) в аргументах ожидает параметры изменения
    изображения разделённые запятой, последовательность действий соответствует
    последовательности введённых аргументов т.е. последовательность -
    s:600x400,resize,s:400x400,crop - сначала задас целевой размер изображения
    600x400, далее скукожит (или растянет изображение в зависимости от
    исходного и других параметров) до 600x400, потом установит новый целевой
    размер 400x400 и отрежет лишнее.

    Аргументы фильтра:
        s:<width>x<height> - указывает целевой размер изображения для
            последующий фенуций
        crop - изменения размера холста до последнего установленного
        resize - изменение размера изображения до последнего установленного
        q:<quality> - укажет качество конечного изображения 0-100
            (используется в последний момент при сохранении)
        c:<color> - цвет заливки для кропа в формате rgba
            (c:255 255 255 255)
        rm:scale|inscribe|stretch - метод ресайза вписать в размер или
            растянуть
        cm:left|center|rigth top|center|bottom - точка отсчёта для кропа

    TO-DO:
        watermark
        определение лица и использование в роли центра изображения
        указание максимальной ширины или высоты
        определение фокуса
    '''

    task_list = args.split(',')
    return get_thumbnail(filename=filename, task_list=task_list)



# -*- coding: utf-8 -*-

from PIL import Image
import logging
import math
import os.path
import re
try:
    from hashlib import md5
except ImportError:
    from md5 import md5

from django.conf import settings

from meringue import settings as wn_settings


class Thumbnail(object):

    crop_method = wn_settings.THUMBNAIL_CROP_MOTHOD
    resize_method = wn_settings.THUMBNAIL_RESIZE_MOTHOD
    quality = wn_settings.THUMBNAIL_QUALITY
    color = wn_settings.THUMBNAIL_COLOR

    store_dir = wn_settings.THUMBNAIL_DIR
    store_url = wn_settings.THUMBNAIL_URL

    def __init__(self, filename, proc='extra@1'):
        if os.path.isabs(filename):
            self.filename = filename
        else:
            self.filename = os.path.join(settings.MEDIA_ROOT, filename)

        if not os.path.isfile(os.path.realpath(self.filename)):
            raise Exception('File \'%s\' does not exist.' % filename)
        self.proc = proc
        if not self.is_valid_thumbnail():
            self.make_thumbnail()

    def is_valid_thumbnail(self):
        '''
            проверяет существование и актуальность превью.
        '''
        if os.path.isfile(self.thumbnail_filename):
            return os.path.getmtime(self.filename) < os.path.getmtime(
                self.thumbnail_filename
            )
        return False

    def get_thumbnail_filename(self):
        if getattr(self, '_thumbnail_filename', None) is None:
            filename = '%s.png' % (self.get_hash(), )
            if not os.path.exists(self.store_dir):
                os.mkdir(self.store_dir)
            self._thumbnail_filename = os.path.join(self.store_dir, filename)
        return self._thumbnail_filename
    thumbnail_filename = property(get_thumbnail_filename)

    def get_hash(self):
        '''
            Указывать конечный размер?? зачем?
        '''
        hash = [i for i in self.proc]  # self.proc
        hash.append(os.path.basename(self.filename))
        return md5(unicode(hash)).hexdigest()

    def make_thumbnail(self):
        try:
            self.image = Image.open(self.filename)
            self.image = self.image.convert('RGBA')
        except IOError, detail:
            raise Exception(detail)

        self.size = [float(i) for i in self.image.size]
        for proc in self.proc:
            try:
                change = getattr(self, '_%s' % proc)
                change()
            except AttributeError:
                method, args = proc.split(':')
                change = getattr(self, '_%s' % method)
                change(args)

        self.image.save(self.thumbnail_filename, 'PNG',
                        quality=self.quality, optimize=1)

    # методы
    def _crop(self):
        thumb = Image.new(mode='RGBA', size=self.new_size, color=self.color)

        if self.crop_method[0] == 'left':
            x = 0
        elif self.crop_method[0] == 'right':
            x = self.new_size[0]-self.size[0]
        else:
            x = (self.new_size[0]-self.size[0])/2

        if self.crop_method[1] == 'left':
            y = 0
        elif self.crop_method[1] == 'right':
            y = self.new_size[1]-self.size[1]
        else:
            y = (self.new_size[1]-self.size[1])/2

        thumb.paste(self.image, (int(x), int(y)))
        self.image = thumb

        self.size = [i for i in self.image.size]

    def _resize(self):
        if self.resize_method == 'scale':
            res = max(min(self.new_size[0], self.size[0])/self.size[0],
                      min(self.new_size[1], self.size[1])/self.size[1])
            new_size = [int(math.floor(self.size[0]*res)),
                        int(math.floor(self.size[1]*res))]
        elif self.resize_method == 'inscribe':
            res = min(min(self.new_size[0], self.size[0])/self.size[0],
                      min(self.new_size[1], self.size[1])/self.size[1])
            new_size = [int(math.floor(self.size[0]*res)),
                        int(math.floor(self.size[1]*res))]
        elif self.resize_method == 'stretch':
            new_size = self.new_size
        else:
            new_size = self.new_size
        self.image = self.image.resize(new_size, resample=Image.ANTIALIAS)
        self.size = [i for i in self.image.size]

    # свойства
    def _cm(self, crop_method):
        self.crop_method = crop_method.split(' ')

    def _rm(self, resize_method):
        self.resize_method = resize_method

    def _s(self, size):
        new_size = re.compile(r'(\d+)x(\d+)')
        self.new_size = new_size.search(size).groups()
        self.new_size = [float(i) for i in self.new_size]
        self.new_size[0] = int(self.new_size[0] if self.new_size[0] else
                               self.new_size[1]*self.size[0]/self.size[1])
        self.new_size[1] = int(self.new_size[1] if self.new_size[1] else
                               self.new_size[0]*self.size[1]/self.size[0])

    def _q(self, quality):
        self.quality = int(quality)

    def _c(self, color):
        tmp = tuple()
        for i in color.split(' '):
            tmp += (int(i), )
        self.color = tmp

    # готовые решения
    def _thumbnail(self):
        pass

    def get_thumbnail_url(self):
        if getattr(self, '_thumbnail_url', None) is None:
            self._thumbnail_url = "%s%s" % (
                self.store_url,
                os.path.basename(self.thumbnail_filename)
            )
        return self._thumbnail_url
    thumbnail_url = property(get_thumbnail_url)


def _dummyimage(task_list):
    if settings.DEBUG:
        task_list.reverse()
        final_size = re.compile(r's:(\d+)x(\d+)')
        for task in task_list:
            try:
                size = final_size.search(task).groups()
                break
            except AttributeError:
                pass
        return u'http://dummyimage.com/%sx%s/9e9e9e/424242.png' % size
    return '%simages/noise.png' % settings.STATIC_URL
    # return '%simages/none.gif' % settings.STATIC_URL


def get_thumbnail(filename, task_list):
    if type(task_list) in [str, unicode]:
        task_list = task_list.split(',')
    url = ''
    if os.path.isfile(filename):
        try:
            thumb = Thumbnail(filename, task_list)
            url = thumb.thumbnail_url
        except Exception, er:
            # TODO 1: log error message cuz noone wishes to debug till here
            # TODO 2: Если в размерах ноль при ошибке нет превьюшки
            logging.error(Exception)
            logging.error(er)
            url = _dummyimage(task_list)
    else:
        logging.error(u'File \'%s\' not find' % filename)
        # находить последний размер и цвет фона
        url = _dummyimage(task_list)
        # return ''
    return url



# -*- coding: utf-8 -*-

import os.path
from hashlib import sha256

from django.core.files import uploadhandler


def _rename(v, rn):
    return u'{0}{1}'.format(sha256(v).hexdigest(), os.path.splitext(rn)[1])


class MemoryFileUploadHandler(uploadhandler.MemoryFileUploadHandler):
    def file_complete(self, file_size):
        self.file_name = _rename(self.file.getvalue(), self.file_name)
        return super(MemoryFileUploadHandler, self).file_complete(file_size)


class TemporaryFileUploadHandler(uploadhandler.TemporaryFileUploadHandler):
    def file_complete(self, file_size):
        self.file_name = _rename(self.file.getvalue(), self.file_name)
        return super(MemoryFileUploadHandler, self).file_complete(file_size)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import meringue

from setuptools import setup, find_packages


long_description = '\n'.join([
    open('README.rst').read(),
    open('CHANGELOG.rst').read()
])

setup(
    name = 'Meringue',
    description = '''Weboven django base package''',
    version = meringue.version,
    license = 'GNU GPL v3',
    author = 'weboven team',
    author_email = 'dd@weboven.net',
    url = 'https://github.com/dd/Meringue',
    install_requires = [
        'Django>=1.5.0',
        'pillow>=2.0.0',
    ],
    packages = find_packages(),
    include_package_data = True,
    zip_safe = True,
    platforms = 'All',
    long_description = long_description,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Natural Language :: Russian',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



# -*- coding: utf-8 -*-


class FileNotFindError(Exception):

    def __init__(self, file):
        message = "Static file \"{file}\" not find".format(file=file)
        super(FileNotFindError, self).__init__(message)



# -*- coding:utf-8 -*-

import os

from django.conf import settings
from django.utils import timezone


########
# HOST #
########

PORT = getattr(
    settings,
    'MERINGUE_SITE_PORT',
    None
)


##################
# current folder #
##################

BASE_DIR = os.path.dirname(os.path.dirname(__file__))
path = lambda *args: os.path.join(BASE_DIR, *args).replace('\\', '/')


##################
# copyright year #
##################

START_YEAR = getattr(
    settings,
    'MERINGUE_START_YEAR',
    timezone.now().year
)


#############
# thumbnail #
#############

THUMBNAIL_CROP_MOTHOD = getattr(
    settings,
    'MERINGUE_THUMBNAIL_CROP_MOTHOD',
    ['center', 'center']
)
THUMBNAIL_RESIZE_MOTHOD = getattr(
    settings,
    'MERINGUE_THUMBNAIL_RESIZE_MOTHOD',
    'inscribe'
)
THUMBNAIL_QUALITY = getattr(
    settings,
    'MERINGUE_THUMBNAIL_QUALITY',
    100
)
THUMBNAIL_COLOR = getattr(
    settings,
    'MERINGUE_THUMBNAIL_COLOR',
    (255, 255, 255, 0)
)
THUMBNAIL_DIR = getattr(
    settings,
    'MERINGUE_THUMBNAIL_DIR',
    os.path.join(settings.MEDIA_ROOT, 'temp')
)
THUMBNAIL_URL = getattr(
    settings,
    'MERINGUE_THUMBNAIL_URL',
    settings.MEDIA_URL + 'temp/'
)


############
# UNIFYING #
############

DEFAULT_PHONE_LOCALIZATION = getattr(
    settings,
    'MERINGUE_DEFAULT_PHONE_LOCALIZATION',
    'RU'
)



# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import models
from django.db.models.query import QuerySet
from django.utils.translation import ugettext_lazy as _
import django.db.models.options as options

if 'django_hosts' in settings.INSTALLED_APPS:
    from django_hosts.resolvers import reverse
else:
    from django.core.urlresolvers import reverse

from . import settings as meringue_settings


options.DEFAULT_NAMES = options.DEFAULT_NAMES + (
    'host_name',
    'view',
    'reverse_args',
)


##########
# mixins #
##########

class GetAbsoluteUrlMixin(object):

    '''
        Миксин автоматизирующий получение абсолютного адреса
        Для получения адреса использует стандартную функцию reverse (в
    случае если установлен django_hosts то reverse_full)

        При реверсе предполагается следующая схема:

            * host_name - название верхнего модуля в нижнем регистре
        (используется в случае установенного django_hosts)

            * view - '<namespace>:<url_name>':
                - namespace - название приложения приведённое к нижнему
            регистру и точки заменённые на нижнее подчёркивание
                - url_name - название класса приведённое к нижнему регистру
            с приставкой '-detail'

            * args - по умолчанию pk модели

        При определении класса модели можно предустановить параметры:
            * host_name - название целевого хоста (string)
            * view - целевой url (string)
            * reverse_args - список атрибутов модели (list)
    '''

    def _host_name(self):
        if not hasattr(self._meta, 'host_name'):
            module_name = self.__module__.split('.')[0]
            module = __import__(module_name)
            self._meta.host_name = getattr(module, 'host_name',
                                           module_name.lower())
        return self._meta.host_name

    def _view(self):
        if not hasattr(self._meta, 'view'):
            namespace = self.__module__[:-7].lower().replace('.', '_')

            cls = self._meta.model_name
            view = '%s-detail' % cls

            self._meta.view = '%s:%s' % (namespace, view)
        return self._meta.view

    def _reverse_args(self):
        reverse_args = getattr(self._meta, 'reverse_args', ['pk'])
        return [getattr(self, key) for key in reverse_args]

    def get_absolute_url(self):
        reverse_args = {
            'viewname': self._view(),
            'args': self._reverse_args(),
        }
        if 'django_hosts' in settings.INSTALLED_APPS:
            reverse_args.update({
                'host': self._host_name(),
                'port': meringue_settings.PORT,
            })
        return reverse(**reverse_args)


#############
# querysets #
#############

class PublishQuerySet(QuerySet):

    def published(self, **kwargs):
        return self.filter(is_published=True, **kwargs)


############
# managers #
############

class PublishManager(models.Manager):
    use_for_related_fields = True

    def get_query_set(self):
        return PublishQuerySet(self.model)

    def published(self, *args, **kwargs):
        return self.get_query_set().published(*args, **kwargs)


##########
# models #
##########

class PublishModel(GetAbsoluteUrlMixin, models.Model):

    '''
        Абстрактная модель с признаком публикации (поле is_published)
        менеджер имеет метод published() аналог стандартного filter() с
    заранее указанными is_published=True
        так же наследует GetAbsoluteUrlMixin
    '''

    is_published = models.BooleanField(
        verbose_name=_(u'публикация'),
        help_text=_(u'Отображать/Скрыть'),
        default=True,
        db_index=True,
    )
    ctime = models.DateTimeField(auto_now_add=True)
    mtime = models.DateTimeField(auto_now=True)

    objects = PublishManager()

    class Meta:
        abstract = True
